package ru.dbotthepony.mc.otm.menu;

import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.inventory.MenuType;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraftforge.fmllegacy.network.PacketDistributor;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.Registry;
import ru.dbotthepony.mc.otm.block.entity.BlockEntityMatterPanel;
import ru.dbotthepony.mc.otm.capability.IPatternStorage;
import ru.dbotthepony.mc.otm.network.MatteryNetworking;
import ru.dbotthepony.mc.otm.network.PatternStateSendListPacket;

import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.List;

public class MatterPanelMenu extends MatteryMenu {
	public MatterPanelMenu(int p_38852_, Inventory inventory) {
		this(p_38852_, inventory, null);
	}

	public MatterPanelMenu(int p_38852_, Inventory inventory, BlockEntityMatterPanel tile) {
		super(Registry.Menus.MATTER_PANEL, p_38852_, inventory, tile);
	}

	@Override
	public void broadcastFullState() {
		super.broadcastFullState();
		sendPatternsToClient();
	}

	private boolean initial_send = false;

	@Override
	public void broadcastChanges() {
		super.broadcastChanges();

		if (!initial_send)
			sendPatternsToClient();
	}

	public ArrayList<IPatternStorage.PatternState> patterns = new ArrayList<>();
	public int changeset = 0;

	public void networkStates(List<IPatternStorage.PatternState> list) {
		changeset++;
		patterns.clear();
		patterns.addAll(list);
	}

	public void requestReplication(ServerPlayer ply, IPatternStorage.PatternState state, int how_much) {
		if (tile == null)
			return;

		var grid = ((BlockEntityMatterPanel) tile).getMatterGrid();

		if (grid == null)
			return;

		if (!grid.hasPatternState(state)) {
			OverdriveThatMatters.LOGGER.error("Received replication request from {} of {}, but can't find it nowhere", ply, state);
			return;
		}

		OverdriveThatMatters.LOGGER.debug("add task: {}", ((BlockEntityMatterPanel) tile).addTask(state.item(), how_much));
	}

	public void sendPatternsToClient() {
		if (inventory.player.level.isClientSide) {
			initial_send = true;
			return;
		}

		if (tile != null) {
			var grid = ((BlockEntityMatterPanel) tile).getMatterGrid();

			if (grid != null) {
				initial_send = true;
				MatteryNetworking.CHANNEL.send(PacketDistributor.PLAYER.with(() -> (ServerPlayer) inventory.player), new PatternStateSendListPacket(containerId, grid.getStoredPatterns()));
			}
		}
	}

	@Override
	protected int getWorkingSlotStart() {
		return 0;
	}

	@Override
	protected int getWorkingSlotEnd() {
		return 0;
	}
}
